@@ -254,6 +254,7 @@ class Configuration(optparse.OptionParser):
         self.config = {}
         self.configuration_providers = configuration_providers
         self.types = types
+        self.added = set() # set of items added to the configuration
 
         # setup optionparser
         if 'description' not in parser_args:
@@ -357,6 +358,7 @@ class Configuration(optparse.OptionParser):
 
     def __call__(self, *args):
         """add items to configuration and check it"""
+        # TODO: configuration should be locked after this is called
 
         # start with defaults
         self.config = self.default_config()
@@ -367,7 +369,9 @@ class Configuration(optparse.OptionParser):
 
         # validate total configuration
         self.validate()
-        # TODO: configuration should be locked after this is called
+
+        # return the configuration
+        return self.config
 
     def add(self, config, check=True):
         """update configuration: not undoable"""
@@ -390,6 +394,8 @@ class Configuration(optparse.OptionParser):
                     raise NotImplementedError
             else:
                 self.config[key] = value
+            self.added.add(key)
+
 
     ### methods for optparse
     ### XXX could go in a subclass
